
#ifndef PLATFORMSOFTWAREKEYBOARD_H
#define PLATFORMSOFTWAREKEYBOARD_H

#include <string>

namespace LibPlatform {

class SoftwareKeyboard {
public:
	typedef enum
	{
		DISPLAY_TV = 0,
		DISPLAY_GAMEPAD0,
		DISPLAY_GAMEPAD1,

		MAX_DISPLAYS
	} Display;

	typedef enum
	{
		MODE_FULL = 0,
		MODE_NUMERIC,
		MODE_QWERTY,
		MODE_HAND,
		MODE_NNID,
		MODE_PASSWORD,

		MAX_MODES
	} Mode;

	typedef enum
	{
		TYPE_QWERTY_JP = 0,
		TYPE_QWERTY_US_ENG,
		TYPE_QWERTY_US_FRA,
		TYPE_QWERTY_US_SPA,
		TYPE_QWERTY_US_POR,
		TYPE_QWERTY_EU_ENG,
		TYPE_QWERTY_EU_FRA,
		TYPE_QWERTY_EU_DEU,
		TYPE_QWERTY_EU_ITA,
		TYPE_QWERTY_EU_SPA,
		TYPE_QWERTY_EU_NLD,
		TYPE_QWERTY_EU_POR,
		TYPE_QWERTY_EU_RUS,
		TYPE_50ON,
		TYPE_CELL_PHONE,
		TYPE_HAND,
		TYPE_SIGN_JP,
		TYPE_SIGN_LATIN,
		TYPE_SIGN_US,

		MAX_TYPES
	} Type;

	typedef enum
	{
		FLAG_QWERTY_JP =		1 << TYPE_QWERTY_JP,
		FLAG_QWERTY_US_ENG =	1 << TYPE_QWERTY_US_ENG,
		FLAG_QWERTY_US_FRA =	1 << TYPE_QWERTY_US_FRA,
		FLAG_QWERTY_US_SPA =	1 << TYPE_QWERTY_US_SPA,
		FLAG_QWERTY_US_POR =	1 << TYPE_QWERTY_US_POR,
		FLAG_QWERTY_EU_ENG =	1 << TYPE_QWERTY_EU_ENG,
		FLAG_QWERTY_EU_FRA =	1 << TYPE_QWERTY_EU_FRA,
		FLAG_QWERTY_EU_DEU =	1 << TYPE_QWERTY_EU_DEU,
		FLAG_QWERTY_EU_ITA =	1 << TYPE_QWERTY_EU_ITA,
		FLAG_QWERTY_EU_SPA =	1 << TYPE_QWERTY_EU_SPA,
		FLAG_QWERTY_EU_NLD =	1 << TYPE_QWERTY_EU_NLD,
		FLAG_QWERTY_EU_POR =	1 << TYPE_QWERTY_EU_POR,
		FLAG_QWERTY_EU_RUS =	1 << TYPE_QWERTY_EU_RUS,
		FLAG_50ON =				1 << TYPE_50ON,
		FLAG_CELL_PHONE =		1 << TYPE_CELL_PHONE,
		FLAG_HAND =				1 << TYPE_HAND,
		FLAG_SIGN_JP =			1 << TYPE_SIGN_JP,
		FLAG_SIGN_LATIN =		1 << TYPE_SIGN_LATIN,
		FLAG_SIGN_US =			1 << TYPE_SIGN_US,
		FLAG_ALL =				FLAG_QWERTY_JP
							  | FLAG_QWERTY_US_ENG
							  | FLAG_QWERTY_US_FRA
							  | FLAG_QWERTY_US_SPA
							  | FLAG_QWERTY_US_POR
							  | FLAG_QWERTY_EU_ENG
							  | FLAG_QWERTY_EU_FRA
							  | FLAG_QWERTY_EU_DEU
							  | FLAG_QWERTY_EU_ITA
							  | FLAG_QWERTY_EU_SPA
							  | FLAG_QWERTY_EU_NLD
							  | FLAG_QWERTY_EU_POR
							  | FLAG_QWERTY_EU_RUS
							  | FLAG_50ON
							  | FLAG_CELL_PHONE
							  | FLAG_HAND
							  | FLAG_SIGN_JP
							  | FLAG_SIGN_LATIN
							  | FLAG_SIGN_US,

		MAX_FLAGS
	} Flag;

	typedef enum
	{
		PASSWORD_DISABLE = 0,
		PASSWORD_IMMEDIATE,
		PASSWORD_1SEC,

		MAX_PASSWORDMODES
	} PasswordMode;

	typedef enum
	{
		INVALIDCHAR_LINEFEED =	1 << 0,
		INVALIDCHAR_SPACE =		1 << 1,
		INVALIDCHAR_ATMARK =	1 << 2,
		INVALIDCHAR_PERCENT =	1 << 3,
		INVALIDCHAR_SLASH =		1 << 4,
		INVALIDCHAR_BACKSLASH =	1 << 5,
		INVALIDCHAR_NUMERIC =	1 << 6,
		INVALIDCHAR_HEART =		1 << 7,
		INVALIDCHAR_EUROMARK =	1 << 8,

		MAX_INVALIDCHARS = 9
	} InvalidCharFlags;

	typedef enum
	{
		INPUTFORMTYPE_MONOSPACE = 0,
		INPUTFORMTYPE_PROPORTIONAL,

		MAX_INPUTFORMTYPES
	} InputFormType;

	typedef enum
	{
		USERENTER_OK = 0,
		USERENTER_CANCEL,
		USERENTER_CHAR,
		USERENTER_BACKSPACE,

		MAX_USERENTERS
	} UserEnter;

	typedef enum
	{
		DISMISS_OK = 0,
		DISMISS_CANCEL,

		MAX_DISMISSOPTIONS
	} DismissOption;

	enum State
	{
		STATE_SHOW_HALFSCREEN = 0,
		STATE_SHOW_FULLSCREEN,
		STATE_SHOW_FULLSCREEN_PAUSE_MEDIA,
        STATE_SHOWING,
		STATE_HIDE,
		STATE_HIDDEN,
		STATE_KEY_DOWN,
		STATE_KEY_UP,
        STATE_SET_TEXT,
		STATE_KEY,
		STATE_OK,
		STATE_CANCEL,

		MAX_STATES
	};

	enum View
	{
		VIEW_NONE,
		VIEW_TV,
		VIEW_GAMEPAD,
		VIEW_UNKNOWN
	};

	class InvokeOptions
	{
	public:
		InvokeOptions() :	display(DISPLAY_GAMEPAD0)
						  , bFullscreen(false)
						  , initialString(L"")
						  , bPrediction(false)
						  , bPauseMedia(false)
						  , mode(MODE_FULL)
						  , flags(FLAG_ALL)
						  , password(PASSWORD_DISABLE)
						  , maxCharacters(-1)
						  , okString(L"")
						  , guideString(L"")
						  , formType(INPUTFORMTYPE_PROPORTIONAL)
						  , invalidChars(InvalidCharFlags(0))
						  , startKeyboard(Type(-1))
						  , startTab(-1)
		{}

		Display display;				// Specifies the display that the software keyboard should be displayed on.
		bool bFullscreen;				// Whether or not to display in full-screen mode.
		std::wstring initialString;		// Specifies the text that will first appear in the software keyboard input field.
		bool bPrediction;				// Whether or not to use predictive input.
		bool bPauseMedia;				// Whether or not to pause any currently-playing media
		Mode mode;						// Keyboard mode (full, numeric, qwerty, etc.).
		Flag flags;						// Keyboard flags for limiting the layouts that can be specified
		PasswordMode password;			// Password mode (disabled, immediate, 1sec).
		int maxCharacters;				// The maximum number of characters the software keyboard will allow (-1 for no limit).
		std::wstring okString;			// The custom string to use for the OK button.
		std::wstring guideString;		// 
		InputFormType formType;			// Character spacing of the input form (only valid in fullscreen mode)
		InvalidCharFlags invalidChars;	// Bitmask for specifying which characters are invalid
		Type startKeyboard;				// The keyboard type that is displayed upon invocation.
		int startTab;					// The keyboard tab that is displayed upon invocation.
	};

	typedef void (*StateChangeCallback)(State, std::wstring characters, View view, int index);

	static void Initialize(void *fs_client, bool isDefaultDisabled);
	static void Deinitialize(void);
	static SoftwareKeyboard *GetInstance(void);

	void AddGlobalCallback(StateChangeCallback callback);
	void RemoveGlobalCallback(StateChangeCallback callback);
	bool HasGlobalCallback(StateChangeCallback callback);
	/**
	@brief Returns whether or not the software keyboard is enabled.
	@return true if the default software keyboard is enabled.
	*/
	bool IsEnabled(void);

	/**
	@brief Returns which screen the software keyboard is being displayed on.
	@return The location that the software keyboard is being displayed on.
	*/
	Display GetDisplay(void);
		
	/**
	@brief Returns whether or not the given cursor position lies within the bounds of the keyboard.
	@param webviewHeight Height of the current webview. Valid values are 1080, 720, 480, and 'WUGP' (special value to be used for GamePad).
	@param cursorY The y-position of the cursor.
	@return true of the given cursor position lies within the bounds of the keyboard.
		    If the keyboard is covered by a sub-window, such as the window for switching keyboards,
			this will return true regardless of where the cursor position lies.
	*/
	bool IsKeyboardTarget(unsigned int webviewHeight, int cursorY);
		
	/**
	@brief Invokes the software keyboard.
	@param display Options for displaying the software keyboard.
	*/
	void Invoke(const InvokeOptions &options);
		
	/**
	@brief Dismisses the software keyboard.
	@param option Specifies whether to treat the dismissal as if the user pressed OK or Cancel.
	*/
	void Dismiss(DismissOption option);

	void DismissOnNextUpdate();

	void SetText(std::wstring newText, int cursorPos = -1);
	void SetSelection(int cursorBegin, int cursorEnd);

	void SetKPADInfoPointer(void *kpad_status_pointer, int channel);
	void SetVPADInfoPointer(void *vpad_status_pointer);

	void SetEventRemote(int eventRemote);

	/**
	@brief Updates the software keyboard.
	*/
	void Update(int channel = 0);
		
	/**
	@brief Is the software keyboard currently being displayed?
	*/
	bool IsDisplaying(void);
		
	/**
	@brief Is the software keyboard currently in fullscreen mode?
	*/
	bool IsFullscreen(void);
		
	/**
	@brief This is a hack to keep the software keyboard drawing in the same rpl as all of its other functions until the software keyboard gets fixed.
	*/
	void DrawTV(void *pColorBuffer);
		
	/**
	@brief This is a hack to keep the software keyboard drawing in the same rpl as all of its other functions until the software keyboard gets fixed.
	*/
	void DrawGamePad(void *pColorBuffer);

    /**
    @brief This is a flag to determine when the validation callback is being run.
    */
    bool mIsValidating;

private:
	SoftwareKeyboard(void *fs_client);
	~SoftwareKeyboard(void);

	void handleOkButton();
	void handleCancelButton();
    void halfscreenDisappearAndCleanup();
};

}

#endif//PLATFORMSOFTWAREKEYBOARD_H
